
#---------------------
# Preprocess MMR data (includes calculation of VR adjustment factors, multiplier)
# Sanqian Zhang and Leontine Alkema, 2015
#---------------------

PrepareMMRdata <- function(
  # location of meta and datall
  metaobject = "data/interm/meta.rda",
  datallobject = "data/interm/datall.rda",
  imputeSElogPM = 0.25 
){
  load(metaobject)  
  load(datallobject)
  
  #---------------------
  # prepare multiplier for VR data 
  
  vrclashinq.d <- rep(FALSE,nrow(datall)) # index to indicate that VR overlaps with CI and should be excluded 
  # (note: is checked for type I/II in loop, and for VR-like at the end)
  
  tCI.Lc <- list()
  
  pstar.ct.long <- pstar.excllate.ct.long <- list() # a list to store multiple entries if necessary
  CYR<-meta$C*meta$nyear ##dimension of ct object in long format 
  for(i in 1:CYR){
    pstar.ct.long[[i]] <- pstar.excllate.ct.long[[i]] <- NA
  }
  
  usa <- pstar <- NULL
  
  datci<-datall[datall$type=="inq" & datall$modelinclude & datall$vr.like==FALSE,]
  nCI<-nrow(datci)
  for(i in 1:nCI){
    ##match based on each entry in the list of CI data
    adj.est.tmp <- adj.est.excl.tmp <- NA # latter excludes late deaths
    country<-as.character(datci$iso[i])
    tstart<-datci$start[i]
    tend<-datci$end[i]
    calyearci.t <-  seq(floor(tstart), ceiling(tend-1))
    
    # any VR-LIKE data in the CI period?
    # exclude if clash (even partially)
    isck3.d<-datall$modelinclude & datall$vr.like==TRUE & datall$iso==country
    isck3.match.CItime.d<-isck3.d[which(
      ((datall$start+datall$end)/2)[isck3.d]>=tstart & ((datall$start+datall$end)/2)[isck3.d]<=tend  )]
    if(length(isck3.match.CItime.d)>0){
      vrclashinq.d[isck3.match.CItime.d]<-TRUE
    }
    
    ## which of data points are of vr type 1 or type 2 and from country C and falls within the period of the CI observation
    # here d refers to index in datall
    # is.element is used because the starting times of VR data are all integers 
    getd.x <- which(datall$type=="vr" & datall$modelinclude & datall$vr.like==FALSE
                    & datall$iso==country & is.element(datall$start,calyearci.t)) 
    if (length(getd.x) == 0){
    } else {
      vrclashinq.d[getd.x] <- TRUE #as long as clash, even just partial, exclude! 
      # calculate multiplier
      CI.env <- datci$final_env[i]
      CI.mat <- datci$final_pm[i]*CI.env 
      if (is.na(CI.env)){
        CI.mat <- datci$obs_mat[i]
        CI.env <- datci$obs_mat[i]/datci$final_pm[i]
      }
      
      # note: the match may be partial for env and vrincllate, ie missing VR country-years during CI, or CI stops/starts in middle of year
      vr.env <- GetSums(start = tstart, end = tend, 
                        tosum.t = datall[getd.x, "final_env"],
                        calyear.t = datall[getd.x, "start"], 
                        na.rm = TRUE)
      vr.mat.incllate <- GetSums(start = tstart, end = tend, 
                                 tosum.t = datall[getd.x, "obs_mat.incl.late"],
                                 calyear.t = datall[getd.x, "start"], 
                                 na.rm = TRUE)
      usability <- mean(datall$usability_vr[getd.x], na.rm= TRUE)
      
      # note: for some countries, eg Sweden, CI overlap between ICD9 and 10
      # for those CIs, we cannot use the info to say anything about ICD10 years so don't include here
      if (sum(is.na(datall[getd.x, "obs_late.mat"])) == sum(is.na(datall[getd.x, "obs_mat.incl.late"]))){
        vr.mat.excllate <- GetSums(start = tstart, end = tend, 
                                   tosum.t = datall[getd.x, "obs_mat.incl.late"] - datall[getd.x, "obs_late.mat"],
                                   calyear.t = datall[getd.x, "start"], 
                                   na.rm = TRUE) 
        
      } else {
        vr.mat.excllate <- NA
      }
      
      adj.est.tmp<-ifelse(vr.mat.incllate==0,NA,GetPMfromME(CI.mat,CI.env)/GetPMfromME(vr.mat.incllate,vr.env))
      adj.est.excl.tmp <-ifelse(vr.mat.excllate==0,NA,GetPMfromME(CI.mat,CI.env)/GetPMfromME(vr.mat.excllate,vr.env))
      
      
      c.index<-which(meta$iso.c==country)
      t.index1 <- Gett.i(min(calyearci.t), year.t = meta$year.t)
      tCI.Lc[[country]] <- c(tCI.Lc[[country]], t.index1)
      
      
      t.index2<-Gett.i(max(calyearci.t), year.t = meta$year.t)
      
      usa <- c(usa, usability)
      pstar <- c(pstar, adj.est.tmp)
      
      for(p in t.index1:t.index2){
        ct1 <- c.index+meta$C*(p-1)
        pstar.ct.long[[ct1]] <- c(pstar.ct.long[[ct1]],adj.est.tmp)
        pstar.excllate.ct.long[[ct1]] <- c(pstar.excllate.ct.long[[ct1]],adj.est.excl.tmp)
        
      }
    }
  } # end CI loop
  
  
  # take means if there is more than 1 adj for a country-period
  mean.pstar.ct.long <- sapply(pstar.ct.long, mean, na.rm=T)
  pstar.incllate.ct <- matrix(mean.pstar.ct.long,nrow=meta$C)
  mean.pstar.excllate.ct.long <- sapply(pstar.excllate.ct.long, mean, na.rm=T)
  pstar.excllate.ct <- matrix(mean.pstar.excllate.ct.long,nrow=meta$C)
  
  save(pstar.incllate.ct, file = "data/interm/tmp_pstar.rda")
  print("Object tmp_pstar written to data/interm/tmp_pstar.rda")
  
  # step 2: get values for all country-years
  # function to interpolate pstar.ct
  interpolate.pstar <- function(pp, 
                                # setting for ICD10 only, note that one of the pstars needs to be nonNA in first 5 years of ICD10
                                icd10 = FALSE, tstarticd10 = NULL){
    pstar<-pp
    #takes in a vector of pstar.ct's with length nyear
    # for ICD10, we first make sure we have the starting point in [t1,t1+4]
    if (icd10){
      tstartperiod <- seq(tstarticd10, min(length(pp), tstarticd10+4))
      if (sum(!is.na(pstar[tstartperiod]))==0){
        print("Stop: you need to define a starting point for ICD10 first!")
        return(NA)
      }
    }
    if(sum(!is.na(pstar))==0){ # applies only to ICD9
      pstar<-rep(1.5,length(pstar))
      output<-pstar
    }else{
      start.pstar<-which(!is.na(pstar))[1]
      end.pstar<-which(!is.na(pstar))[length(which(!is.na(pstar)))]
      
      missing<-which(is.na(pstar))[which(is.na(pstar))>start.pstar & which(is.na(pstar))<end.pstar]
      for(i in missing){
        i.b4<-max(which(!is.na(pstar))[which(!is.na(pstar))<i])
        i.aft<-min(which(!is.na(pstar))[which(!is.na(pstar))>i])
        pstar[i]<-pstar[i.b4]+(i-i.b4)*(pstar[i.aft]-pstar[i.b4])/(i.aft-i.b4)
      }
      
      if(end.pstar!=length(pstar)){
        pstar[(end.pstar+1):length(pstar)]<-rep(pstar[end.pstar],length(pstar)-end.pstar)
      }
      if(start.pstar!=1){
        if (!icd10){
          for(i in 1:(start.pstar-1)){
            ifelse(pstar[start.pstar]<1.5,         
                   pstar[i]<-1.5+ifelse(start.pstar-i>5,0,(pstar[start.pstar]-1.5)/5*(5-(start.pstar-i))),
                   pstar[i]<-pstar[start.pstar])
          }
        } else {# keep constant, used only from first icd10 year, imputation for that t only occurs if a country has CI in first 5-year period but not first year
          pstar[1:(start.pstar-1)]<-pstar[start.pstar]
        }
      }
      output<-pstar
    }
    return(output)
  }
  
  pstar.ct <- matrix(NA, meta$C, meta$nyears) 
  for( c in 1:meta$C){
    pstar.ct[c,]<-interpolate.pstar(pstar.incllate.ct[c,])
  }
  pstar.incl.late.ct <- pstar.ct
  
  # for ICD10 coding, exclude late maternal deaths
  # select only those countries where VR data excludes late from a given year onwards (for some, the added VR data does not exclude late)
  icd10countries <- Getc.i(paste(unique(datall$iso[which(datall$late.excluded & datall$modelinclude)])), meta$iso.c) 
  
  for(c in icd10countries){
    t.icd10 <- Gett.i(datall[which(paste(datall$iso)==meta$iso.c[c] & datall$modelinclude & datall$late.excluded),"year"]-0.5,meta$year.t)
    tstart.icd10 <- min(t.icd10)
    tend.icd10 <- max(t.icd10)
    t.icd10 <- seq(tstart.icd10, tend.icd10)
    
    # make sure we have at least one non-NA in (tstart.icd10, min(maxtobserved, tstart.icd10+4)), if not, calculate it based on pstar_incllate:
    tstartperiod <- seq(tstart.icd10, min(tstart.icd10+4,tend.icd10))
    # this is adjusted for countries with additional info, see below
    if (sum(!is.na(pstar.excllate.ct[c,tstartperiod]))==0){
      # end the start period at min(oldest ICD10-CI (if any) -5, previously defined end point)
      country <- paste(meta$iso.c[c])
      tCIinICD10 <- NULL
      if (is.element(country, names(tCI.Lc))) tCIinICD10 <- tCI.Lc[[country]][tCI.Lc[[country]] >= tstart.icd10]
      if (!is.null(tCIinICD10)){
        firsttCIinICD10 <- min(tCIinICD10)
        # note that CI is at least in ticd10+6 so -5 means start period will also include start year
        tstartperiod <- seq(tstart.icd10, min(max(tstartperiod), firsttCIinICD10 - 5))
        # make sure that for countries without ICD9 CI(s) to support the starting value but with ICD10 CI,
        # that we keep the start values constant up to (first CIstartyear-5)
        # for getting start for icd9, 5 years is the default
        if (sum(tCI.Lc[[country]]<tstart.icd10) == 0 & sum(tCI.Lc[[country]]>= tstart.icd10) > 0 ){# no CI in icd9 years, CI in icd10 years
          tmaxtokeepconstant <- min(tCI.Lc[[country]]) - 5
        }
      } else {
        tmaxtokeepconstant <- max(tstartperiod)
      }
      
      # note: datall$late.excluded is NA for non-VR
      # divide total maternal death from vr.incllate*pstart by total maternal deaths excl late from vr
      # sequence with NAs if missing years
      vrincllate <- vrexcllate <- rep(NA, length(tstartperiod))
      for (t in 1:length(tstartperiod)){
        index <- tstartperiod[t]
        select <- which(paste(datall$iso)==meta$iso.c[c] & datall$modelinclude & datall$late.excluded &
                          (datall$year -0.5)==meta$year.t[index])
        if (length(select)==1){
          vrincllate[t] <- datall[select,"obs_mat.incl.late"]
          vrexcllate[t] <- datall[select,"obs_mat"]
        }
      }
      # there has to be at least one NA given the how the startperiod is defined (needs to start with an icd10 observation)
      # but could be all zeroes (eg iceland and qatar).
      # if all zero, use the one for late
      if (sum(vrexcllate, na.rm = T)==0){
        pstar.excllate.ct[c,tstartperiod] <- pstar.incl.late.ct[c,tstartperiod]
      } else {
        pstar.excllate.ct[c,tstartperiod] <- (
          sum(pstar.incl.late.ct[c,tstartperiod]*vrincllate, na.rm = T)
          /sum(vrexcllate, na.rm = T))
      }
      # make sure that for countries without ICD9 CI(s) to support the starting value but with ICD10 CI,
      # that we keep the start values constant up to (first CIstartyear-5)
      if (tmaxtokeepconstant > max(tstartperiod))  pstar.excllate.ct[c,(max(tstartperiod)+1):tmaxtokeepconstant] <- pstar.excllate.ct[c,max(tstartperiod)]
    }
    pstar.excllate.ct[c,] <- interpolate.pstar(pstar.excllate.ct[c,], icd10 = TRUE, tstarticd10 = tstart.icd10)
    pstar.ct[c,t.icd10] <- pstar.excllate.ct[c,t.icd10]
    if (meta$nyears > tend.icd10) pstar.ct[c, (tend.icd10+1):meta$nyears] <- NA
  }

  # get the multipliers into datall
  isjork3.d<- datall$type=="vr"|datall$vr.like==TRUE
  getc.jork3<-Getc.i(as.character(datall$iso[isjork3.d]),meta$iso.c)
  gett.jork3<-Gett.i(floor(datall$start[isjork3.d]),meta$year.t)
  multiplier.d<-rep(NA,nrow(datall))
  for(i in 1:length(getc.jork3)){
    multiplier.d[isjork3.d][i]<-pstar.ct[getc.jork3[i],gett.jork3[i]]
  }
  datall$multiplier<-multiplier.d
  
  datall$modelinclude[vrclashinq.d]<-FALSE
  datall$exclude.reason[vrclashinq.d]<-"vrisci"
  
  write.csv(pstar.ct, file = paste0("data/extra/calculatedVRmultiplier_datall.csv"),row.name=meta$name.c)
  
  datall$exclude.reason[datall$modelinclude]<-NA
  
  data.exclude.reason.long <- rep(NA, length(datall$exclude.reason))
  data.exclude.reason.long[datall$exclude.reason =="pre1985"] <- "Data or reference period before 1985"
  data.exclude.reason.long[datall$exclude.reason =="subnat"] <- "Data has subnational or subpopulational coverage"
  data.exclude.reason.long[datall$exclude.reason =="vrisci"] <- "The VR observation period overlaps with a reference period of a higher quality data source (special study such as an inquiry or surveillance data)"
  data.exclude.reason.long[datall$exclude.reason =="lowusa"] <- "Usability of VR data is too low (<60%)"
  data.exclude.reason.long[datall$exclude.reason =="lowusawindow"] <- "VR data in a window with data of low usability"
  data.exclude.reason.long[datall$exclude.reason =="noenv"] <- "No deaths envelope found for specialized study"
  data.exclude.reason.long[datall$exclude.reason =="mmeigcheck"] <- "MMEIG data assessment"
  data.exclude.reason.long[datall$exclude.reason =="noinfo"] <- "Data contains no usable information on PM and MMR"
  data.exclude.reason.long[datall$exclude.reason =="noage"] <- "No age breakdown in reporting of VR"
  data.exclude.reason.long[datall$exclude.reason =="HIV"] <- "MMEIG data assessment: AIDS miscoding"
  data.exclude.reason.long[datall$exclude.reason =="vrdataissue"] <- "Change of data collection scheme leading to inconsistency with more recent VR data"
  
  datsum<-data.frame(country=meta$name.c[Getc.i(as.character(datall$iso),meta$iso.c)]
                     ,datall[,c("iso","start","end","type","vr.like","definition","obs_env","obs_mat","obs_pm","obs_mmr")],
                     VR.multiplier=datall$multiplier,
                     datall[,c("usability_vr","compl_vr", "ill_vr", "modelinclude")], 
                     exclude.reason=data.exclude.reason.long,
                     datall[,c("datasource","datareference")])
  print("A summary of data is saved to data/extra/data_overview_before1f.rda")
  save(datsum, file = paste0("data/extra/datsum_before1f.rda"))
  write.csv(datsum, file = paste0("data/extra/data_overview_before1f.csv"))
  
  
  #---------------------
  # adjust for VR entries with 0 observed maternal deaths by pooling the observation with neighbouring observations: 
  
  # redistribute envelopes to surrounding years
  # where surrounding period are calendar years ref year+/t, where we try t from 1 to 30 and use smallest x with a match
  # we then recalculate the reference years as the envelope-weighted average
  # to update ref years:
  # relabel year into year_obs (we need to keep it to avoid confusion)
  # then recalculate years
  
  datall2 <- datall
  datall2$obs_year <- datall2$year
  # note: modelinclude may be based on excl because of CI or because of usa
  is0 <- datall$type=="vr" & datall$modelinclude & datall$obs_mat==0
  select0s <- which(is0)
  datall2$exclude0 <- FALSE
  length(select0s)
  i=select0s[1]
  nomatch <- changedpm <- NULL
  for (i in select0s){
    iso <- datall$iso[i]
    year <- datall$year[i]
    # here we don't use modelinclude, we redistribute env deaths also to years 
    # that are excluded from the model if they are closest to the obs
    # eg. to years with CIs
    selectcountryobs <- datall$type=="vr" & datall$include & datall$obs_mat!=0 & datall$iso==iso 
    # use the potentially updated reference years stored in datall2 to figure out whether there are obs in period around ref year
    years <- datall2$year[selectcountryobs]
    keepfindingmatch <-  TRUE
    t <- 1
    while (keepfindingmatch & (t < 30)){
      s <- sum(is.element(floor(seq(year-t, year+t)), floor(years))) # are there any obs in period around year?
      if (s>0){
        for (j in (which(selectcountryobs))[is.element(floor(years), floor(seq(year-t, year+t)))]){ 
          env_new <- datall2$final_env[j] +datall2$final_env[i]/s
          # ref year is weighted average, change before changing datall2$env
          datall2$year[j] <- (datall2$final_env[i]/s*year +  datall2$final_env[j]*datall2$year[j])/env_new
          
          datall2$final_env[j] <- env_new
          changedpm <- c(changedpm,j)
        } 
        datall2$exclude0[i] <- TRUE     
        keepfindingmatch <- FALSE
      } else {
        t <- t+1
      } # end trial for t
    }
    if (t==30){
      nomatch <- c(nomatch, i)
    }
  }
  datall2$exclude0[nomatch] <- TRUE
  datall2$modelinclude[datall2$exclude0] <- FALSE
  
  select <- changedpm[datall$modelinclude[changedpm]]
  datall2$final_pm[select] <- (datall$final_pm*datall$final_env/datall2$final_env)[select]
  getc.s<-Getc.i(as.character(datall2$iso[select]),meta$iso.c)
  gett.s<-Gett.i(floor(datall2$year[select]),meta$year.t)
  for(j in 1:length(select)){
    i<-select[j]
    datall2$final_mmr[i]<-datall2$final_pm[i]*meta$deaths.ct[getc.s[j],gett.s[j]]/meta$births.ct[getc.s[j],gett.s[j]]
  }
  
  # check if ref years changed
  (datall2$year - datall2$obs_year)[changedpm]
  (datall2$year - datall2$obs_year)[select]
  (floor(datall2$year) - floor(datall2$obs_year))[select]
  datall2$iso[which((floor(datall2$year) - floor(datall2$obs_year))>1)]
  meta$name.c[is.element(meta$iso.c, datall2$iso[which((floor(datall2$year) - floor(datall2$obs_year))>0)])]
  meta$name.c[is.element(meta$iso.c, datall2$iso[which((floor(datall2$year) - floor(datall2$obs_year))>1)])]
  
  
  pm.changed<-rep(0,nrow(datall2))
  pm.changed[changedpm]<-1
  
  # write csv to interm because used for plots
  write.csv(data.frame(datall,adjustedfor0pm=pm.changed,exclude0=datall2$exclude0), 
            file = paste0("data/extra/dat_aft1f_inforelatedtoadj0PMobs.csv"))
  print("Csv file dat_aft1f_inforelatedtoadj0PMobs written to data/interm/dat_aft1f.csv")
  
  datall <- datall2
  
  # ----------------------------
  # approximate stochastic errors for VR and specialized studies
  # construct a new entry modeled_selogpm with sampling and stochastic errors, incl imputed values 
  
  # a. calculate SEs based on stochastic uncertainty for VR and for inquiries, store in stoch_selogpm
  # b. temporarily store combi of observed stoch and sampling errors in final_selogpm 
  # c. get modeled SEs, which is final_selogpm but with imputed values for the NAs
  # imputation is based on max(imputedvalue, max(SE) in country for nonVR/inq (so currently based on DHS only))
  
  datall$stoch_selogpm <- datall$final_selogpm <- rep(NA, dim(datall)[1])
  select <- datall$type=="vr" | datall$type =="inq"
  datall$stoch_selogpm[select] <- GetSElogPM(pm = datall$final_pm[select], env = datall$final_env[select])
  datall$final_selogpm[select] <- datall$stoch_selogpm[select]
  # for other types, use observed SE if available
  datall$final_selogpm[!select] <- datall$obs_selogpm[!select]
  #sum(is.na(datall$stoch_selogpm[datall$type =="vr"& datall$modelinclude]))
  #sum(is.na(datall$stoch_selogpm[datall$type =="inq" & datall$modelinclude]))
  #datall$iso[which(is.na(datall$stoch_selogpm) & datall$type =="inq" & datall$modelinclude)] 
  
  datall$modeled_selogpm <- datall$final_selogpm
  # get SEs within country from nonvr/inq SEs
  select <- !is.element(datall$type, c("vr", "inq"))
  input <- datall$obs_selogpm[select]
  names(input) <- datall$iso[select]
  # get max SE for each country:
  res <- tapply(input, datall$iso[select], max, na.rm =TRUE)#ean, trim = 0, na.rm = T)
  # warnings are about -Inf
  
  #impute in missing non-vr entries (incl inquiries)
  indicestoimpute <- is.na(datall$final_selogpm) & !is.element(datall$type, c("vr"))
  obsSE <- res[paste(datall$iso[indicestoimpute])]
  datall$modeled_selogpm[indicestoimpute] <- ifelse(obsSE > imputeSElogPM, obsSE, imputeSElogPM)
  datall$modeled_selogpm <- ifelse(is.na(datall$modeled_selogpm), imputeSElogPM, datall$modeled_selogpm)
     
  # remove final se
  datall$final_selogpm <- NULL
  
  save(datall, file = paste0("data/interm/datall.rda"))
  print("Object datall updated in data/interm/datall.rda")
  write.csv(datall, file = paste0("data/extra/datall_usedinmodeling.csv"))
  
  print("Warnings related to Inf and -Inf can be ignored") 
  
}