
#---------------------
# Write JAGS model
# Leontine Alkema and Sanqian Zhang, 2015
#---------------------

WriteJagsModel <- function(
  runname = "test",
  JAGSmodel.name = "model.txt",
  AddARMA = TRUE
){
#----------------------------

  output.dir <- MakeDirs(runname)
  
  
# start model file
cat("model {", sep="", append=FALSE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

cat(" 
 #----------------------------
  # data models
    
    for (g in 1:Gtrain){
      logpm.j[getj.gtrain[g]] ~ dnorm(mean.j[getj.gtrain[g]], tau.j[getj.gtrain[g]])
    }

    for (g in 1:G){
      logpmpred.g[g] ~ dnorm(mean.j[getj.g[g]], tau.j[getj.g[g]])
    }
    

    for(g in 1:Ginqtrain){
      logpm.jinq[getjinq.ginqtrain[g]] ~ dnorm(mean.jinq[getjinq.ginqtrain[g]], tau.jinq[getjinq.ginqtrain[g]])
    }

    for(g in 1:Ginq){
      logpmpred.ginq[g] ~ dnorm(mean.jinq[getjinq.ginq[g]], tau.jinq[getjinq.ginq[g]])
    }

    
    for (g in 1:Gnewtrain){
      logpm.jnew[getjnew.gnewtrain[g]] ~ dnorm(mean.jnew[getjnew.gnewtrain[g]], tau.jnew[getjnew.gnewtrain[g]])
    }

    for (g in 1:Gnew){
      logpmpred.gnew[g] ~ dnorm(mean.jnew[getjnew.gnew[g]], tau.jnew[getjnew.gnew[g]])
    }
    
    # ---------------------------
    
    # VR data
    for (j in 1:J){
      mean.j[j] <- log(pvr.j[j])
      pmtrue.j[j] <- R.ct[getc.j[j], gett.j[j]]
      pvr.j[j] <- pmtrue.j[j]/p.j[j]
    }
    
    #----
    # data source 3: incl misc, vr-like and inquiries w/o envelopes

    nonsamplingdhs.se ~ dunif(0.1, 0.5)
    nonsamplingnondhs.se ~ dunif(0.1, 0.5)

    for (j in 1:Jnew){
      tau.jnew[j] <- 1/(1/tausamp.jnew[j] #+isvrlike.jnew[j]*stderrordelta^2
                       + ismisc.jnew[j]*
                  (isdhs.jnew[j]*nonsamplingdhs.se^2+(1-isdhs.jnew[j])*nonsamplingnondhs.se^2))
      mean.jnew[j] <- (logphi.jnew[j]      + 
          log(nonaidsmatoverall.jnew[j]*
          ( (1-ispreg.jnew[j]) + phi.preg.jnew[j]*ispreg.jnew[j] ) 
          + uaids*aidsva.jnew[j] #for all, total for maternal
          +(1-uaids)*aidsva.jnew[j]*ispreg.jnew[j]
          ))
      # for AIDS, reference year is used (instead of reference period)
      #aidsva.jnew[j] <- v.ct[getc.jnew[j],gett.jnew[j]]*a.ct[getc.jnew[j],gett.jnew[j]]
      phi.preg.jnew[j] <- 1/oneminpi.c[getc.jnew[j]]
      logphi.jnew[j] <- log(1/(ismisc.jnew[j]*1.1 + isinq.jnew[j]*1 + isvrlike.jnew[j]*gamma.k3[getk3.jnew[j]]))
    }

",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

cat("
    # VRmultiplier 
    for(g in 1:K2){
      p.j[getj.k2[g]] <- p.wprior.k2[g]
      p.wprior.k2[g] <- max(pstar.j[getj.k2[g]],p.prior.k2[g])
      p.prior.k2[g] ~ dunif(min(1,pstar.j[getj.k2[g]]), delta.upperbound.j[getj.k2[g]])
    }
    for(g in 1:K1){
      p.j[getj.k1[g]]<-pstar.j[getj.k1[g]]
    }
  
    ",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)



#----------------------------------
cat("  
    # specialized studies
    for (j in 1:Jinq){
      mean.jinq[j] <- log(matoverall.jinq[j])
      matoverall.jinq[j] <- (
        inprod(partialwhoenv.xjinq[1:X.jinq[j],j], R.ct[getc.jinq[j], gettstart.jinq[j]:gettend.jinq[j]])/
            sum(partialwhoenv.xjinq[1:X.jinq[j],j])
          ) 
    }

  #----------------------------------
  # data sources 3 (misc) 
  for (j in 1:Jnew){
    nonaidsmatoverall.jnew[j] <- (
      inprod(partialwhoenv.xjnew[1:X.jnew[j],j], nonaidsmatoverall.ct[getc.jnew[j], gettstart.jnew[j]:gettend.jnew[j]])/
        sum(partialwhoenv.xjnew[1:X.jnew[j],j])
      #partialenv = length of interval*E.ct
    ) 
  }
 
  for (c in 1:C){
    oneminpi.c[c] ~ dnorm(oneminpi.hat.c[c], 1/0.05^2)T(0,1)
    oneminpi.hat.c[c] <- isssa.c[c]*0.9 + (1-isssa.c[c])*0.85
  }

  for(g in 1:K3){
    gamma.k3[g] <- max(initialvrmultiplier.k3[g], p.prior.k3[g])
    p.prior.k3[g] ~ dunif(min(initialvrmultiplier.k3[g],1),3)
  }



#------
",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

#----------------------------------
# ARMA part
if (AddARMA){
  cat("  
  #--------------------------
  # ARMA part

  theta ~dunif(-1,0)
  phi ~ dunif(0,1)
  sqrtgamma0 ~ dunif(0,0.025)#sqrtgamma0max)
  tau.lambda <- pow(sigma.lambda,-2)
  sigma.lambda ~ dunif(0,max.sigma.lambda)
  for (c in 1:C){
    gamma0.c[c] <- pow(sqrtgamma0.c[c],2)
    tau.ar.c[c] <- pow(sigma.ar.c[c],-2)

    arma1.c[c] ~ dnorm(0, 1/(gamma0.c[c]))
    arma.ct[c,1] <- arma1.c[c]
    e.ct[c,1] ~ dnorm(sigma.ar.c[c]^2/gamma0.c[c]*arma.ct[c,1], 1/(sigma.ar.c[c]^2*(1-sigma.ar.c[c]^2/gamma0.c[c])))   
    for (t in 2:nyears){
      e.ct[c,t] ~ dnorm(0, tau.ar.c[c])
      arma.ct[c,t] <- phi*arma.ct[c,t-1] - theta*e.ct[c,t-1] + e.ct[c,t]
    }
    sigma.ar.c[c] <- sqrt(gamma0.c[c] /((1-2*phi*theta + theta^2)/(1-phi^2)))
    sqrtgamma0.c[c] <- sqrtgamma0*(1+lambda.c[c])
    lambda.c[c] ~ dnorm(0, tau.lambda)T(-1,2)
  }
 ",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)   
} else {
  cat("  
  arma.ct[1:C,1:nyears] <- input0.ct[1:C,1:nyears]
  ",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)
}

# multilevel model for PM^NA
cat("  
  #----------------------------------
  # multilevel model for PM^NA
  for (c in 1:C){
    for (t in 1:nyears){
      # note negative and positive signs
      logRstarnonaids.ct[c,t] <- (- beta.h[1]*X.cth[c,t,1] + beta.h[2]*X.cth[c,t,2] - beta.h[3]*X.cth[c,t,3]
                              + alpha.c[c] 
          )
      input1.ct[c,t] ~ dinterval(exp(logRstarnonaids.ct[c,t]), vec)
    }
    alpha.c[c] ~ dnorm(alpha.r[getr.c[c]], tau.country)
  } # end C-loop
  vec[1] <- 0.0001 
  vec[2] <- 0.9999
  tau.country <- pow(sigma.country, -2)

",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

#----------------------------------
cat("  
  for (r in 1:R){
    alpha.r[r] ~ dnorm(alpha.world, tau.region)
  }
  alpha.world ~ dnorm(log(0.001), 0.01)
  tau.region <- pow(sigma.region, -2)
  sigma.region ~ dunif(0, 5)
  
  sigma.country ~ dunif(0, 5)
  #---
",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

cat("  
  for (h in 1:3){
    beta.h[h] ~ dnorm(0.5, 0.001)
  }
  #---
",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

#----------------------------------
# putting things together
cat("  
  # putting things together
  for (c in 1:C){
    for (t in 1:nyears){
      Rstarnonaids.ct[c,t] <- exp(logRstarnonaids.ct[c,t])
      munonaidsstar.ct[c,t] <- Rstarnonaids.ct[c,t]/B.ct[c,t]*E.ct[c,t]*(1-a.ct[c,t])
      logmunonaidsstar.ct[c,t]<-log(munonaidsstar.ct[c,t])
    }
  }
",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

cat("  
   #----------------------------------
 for (c in 1:C){
    for (t in 1:nyears){
      #muaids.ct[c,t] <- Raids.ct[c,t]/B.ct[c,t]*a.ct[c,t]*E.ct[c,t]
      mustar.ct[c,t] <- munonaidsstar.ct[c,t] + muaids.ct[c,t]
      
      munonaids.ct[c,t]<-exp(logmunonaids.ct[c,t])
      mu.ct[c,t] <-munonaids.ct[c,t] + muaids.ct[c,t]
      R.ct[c,t] <- mu.ct[c,t]*B.ct[c,t]/E.ct[c,t]
      nonaidsmatoverall.ct[c,t] <- (mu.ct[c,t]-muaids.ct[c,t])*B.ct[c,t]/E.ct[c,t]

    } 
    logmunonaids.ct[c,year.ref]<-logmunonaidsstar.ct[c,year.ref]
    for (t in (year.ref+1):nyears){
      logmunonaids.ct[c,t] <- (logmunonaids.ct[c,t-1]+arma.ct[c,t-1]
                +logmunonaidsstar.ct[c,t]-logmunonaidsstar.ct[c,t-1])
    }
  }# end c loop, before ref year is further below in code

  for (c in 1:C){
    for (t in 1:nyears){
      input1again.ct[c,t] ~ dinterval(R.ct[c,t], vec)
    }
  }

",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

  # loop to get results for t < reference year (relevant only when reference year > 1985)
  cat("  
    for (c in 1:C){
      for (t in 1:(year.ref-1)){
      logmunonaids.ct[c,t] <- (logmunonaids.ct[c,t+1]-arma.ct[c,t] 
              - (logmunonaidsstar.ct[c,t+1]-logmunonaidsstar.ct[c,t]))
      }
    } # end c-loop
    
",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

#----------------------------------
# end model
cat("  
#----------------------------------
} # end model
",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)
  
} # end function
  

# The End!

#-------------------------------
# comments

# to add something
# cat("  
#  BLAbla
# ",sep="", append=TRUE, file=file.path(output.dir, JAGSmodel.name), fill=TRUE)

# on interval:
# add in restrictions
#y[j] ~ dinterval(t[j], lim[j,]) 
#y[j] = 0 if t[j] <= lim[j,1] 
#y[j] = 1 if lim[j,1] < t[j] <= lim[j,2]
#y[j] = 2 if lim[j,2] < t[j] 
