
#----------------------------------------------------------
# Trends in Maternal Mortality: 1990 to 2015
# UN MMEIG (WHO, UNICEF, UNFPA, The World Bank and UNPD) estimates
# Supplementary BMat model code, to reproduce the estimates

# Code written by Leontine Alkema and Sanqian Zhang
# For a technical write-up of the BMat model, see the working paper at http://arxiv.org/abs/1511.03330

#-------
# Overview of steps in this master script:
## part 1: prepare BMat input files from input csvs
## part 2: Run BMat
## part 3: Get country output (and save as csvs)

# Notes:
## This master script is part of a zip-file with R-functions and data inputs (all are needed to run this script)
## You need to install JAGS (it's free, get it here http://mcmc-jags.sourceforge.net/)
## and the R-packages listed below.

#---------------------
# START: THIS PART NEEDS TO BE RAN FOR ANY TASK
#---------------------
## Set working directory to the folder that contains the downloaded public release file, e.g. 
setwd("C:/Users/lalkema/Dropbox/MMR2015_Sanqian/BMat_PublicRelease")

#-----
# load libraries
library(rjags)
library(R.utils)
library(RColorBrewer)
library(msm)
library(mvtnorm)
library(gtools)
library(R2jags)
library(foreign)
# Note: additional libraries are loaded when choosing the (non-default option) "run.on.server=TRUE" in the doMCMC part.

#-------
# read in functions
Rfiles <- list.files(file.path(paste0(getwd(),"/code/functions/")), ".R")
Rfiles <- Rfiles[grepl(".R", Rfiles)]
sapply(paste0(paste0(getwd(),"/code/functions/"), Rfiles), source) 
# gives lots of R output in console, is fine (as long as there is no error reported in red)

#-------
# user option:
# choose a runname 
runname <- "test"


#---------------------
# Part 1: construct BMat input data
# (this part needs to be run once)
#---------------------
# data from data/input are read in
# info is printed on which files are saved where
ReadCovariates()
ReadMMRdata()
PrepareMMRdata()

#---------------------
# Part 2: run the MMR model
# (this part needs to be run once)
#---------------------
SetUpOutputDir(runname = runname)
# Creates an output directory (output/runname) that will contain modeled estimates, and some information related to the input data (meta with country and covariate info and data objects)
GetJagsData(runname = runname) # adds object jagsdata to the outputdir with info to run the model
WriteJagsModel(runname = runname) # adds the JAGS model file to the outputdir
RunMCMC(runname = runname, runsettings = "test") 
#RunMCMC(runname = runname, runsettings = "quick")
#RunMCMC(runname = runname, runsettings = "long")
# for run settings, see documentation for RunMCMC
# choose from ("test", "quick", "long"), where
## test is only for checking if the model runs (just a few iterations); 
## quick is for getting approximate results in an hour or so; 
## long is based on the settings that were used to obtain the MMEIG estimates
CheckConverge(runname = runname) # to check MCMC diagnostics

#-------------------
# Part 3: work with model output
#-------------------
# what results do you want to obtain?
# defaults:
percentiles <- c(0.1, 0.5, 0.9) 
percnames <- paste0("q",percentiles)
GetCountryResults(runname = runname) 
# Results written to csvs. 
# Note: CSVs contain estimates without rounding/multiplications, 
# e.g. mmr is number of maternal deaths per birth (as opposed to rounded and per 100,000 births)
# csv "arrscts.csv" refers to the continuous ARR for various periods, and differences between periods. 

#-------------------
# The End!



