####################  Compute age-adjusted PMs from sisterhood data  #################

# Read input data, create data array for use in computing PMDFs
#Update pathname here
sisterhood.input.df <- read.csv("../sisterhood.input.data.csv")

varnames <- unique(as.character(sisterhood.input.df[,"variable"]))
ages <- paste(seq(15,45,5),seq(19,49,5),sep="-")
xages <- paste("x",seq(15,45,5),".",seq(19,49,5),sep="")
tmp <- sisterhood.input.df[,"variable"]=="survey.pop.pct"
ccodes       <- as.character(sisterhood.input.df[tmp,"iso.code"   ])
survey.names <- as.character(sisterhood.input.df[tmp,"survey.name"])
survey.years <- as.character(sisterhood.input.df[tmp,"survey.year"])
survey.ids <- paste(ccodes,survey.names,survey.years)
pmdf.calc.sisterhood <- array(NA,dim=c(length(survey.ids),length(ages),length(varnames)),
	dimnames=list(survey.ids,ages,c(varnames)))
for (varname in varnames) 
	pmdf.calc.sisterhood[,,varname] <- as.matrix(sisterhood.input.df[sisterhood.input.df[,"variable"]==varname,xages])


# Compute interval midpoints for PMDFs (average of midpoints for adult and maternal mortality intervals)
# Adult mortality
adult.start.dates <- sisterhood.input.df[sisterhood.input.df[,"variable"]=="adult.mort.rate","v.start.date"][tmp]
adult.end.dates   <- sisterhood.input.df[sisterhood.input.df[,"variable"]=="adult.mort.rate","v.end.date"][tmp]
# Maternal mortality
maternal.start.dates <- sisterhood.input.df[sisterhood.input.df[,"variable"]=="maternal.mort.rate","v.start.date"][tmp]
maternal.end.dates   <- sisterhood.input.df[sisterhood.input.df[,"variable"]=="maternal.mort.rate","v.end.date"][tmp]
# Assume missing start dates are 7 years prior to end dates (the latter are the data collection midpoint)
adult.start.dates    <- ifelse(is.na(adult.start.dates   ),adult.end.dates   -7,adult.start.dates   )
maternal.start.dates <- ifelse(is.na(maternal.start.dates),maternal.end.dates-7,maternal.start.dates)
# Compute mid-dates
adult.mid.dates   <- (adult.start.dates+adult.end.dates)/2
maternal.mid.dates   <- (maternal.start.dates+maternal.end.dates)/2

# Set PM dates equal to maternal mortality dates
pmdf.start.dates <- maternal.start.dates
pmdf.end.dates <- maternal.end.dates
pmdf.mid.dates <- maternal.mid.dates
names(adult.start.dates) <- names(adult.end.dates) <- names(adult.mid.dates) <- names(maternal.start.dates) <- 
	names(maternal.end.dates) <- names(maternal.mid.dates) <- names(pmdf.start.dates) <- names(pmdf.end.dates) <- 
	names(pmdf.mid.dates) <- survey.ids
# Note surveys where maternal and adult start dates differ
survey.ids[maternal.start.dates!=adult.start.dates]


# Compute total PMs
pmdf.sisterhood <- rep(NA,length(survey.ids))
names(pmdf.sisterhood)  <- survey.ids
for (survey.id in survey.ids) {
	pmdf.sisterhood[survey.id] <- sum(pmdf.calc.sisterhood[survey.id,,"survey.pop.pct"] * pmdf.calc.sisterhood[survey.id,,"maternal.mort.rate"]) / sum(pmdf.calc.sisterhood[survey.id,,"survey.pop.pct"] * pmdf.calc.sisterhood[survey.id,,"adult.mort.rate"   ]) }
	
pmdf.sisterhood.na <- rep(NA, length(survey.ids))

##################  Create (partial) PMDF estimates file based on sisterhood data  ####################

col1  <- substr(survey.ids,1,3)
#col2  <- cnames[col1]
col2 <- NA
col3  <- pmdf.start.dates
col4  <- pmdf.mid.dates
col5  <- pmdf.end.dates
col5a <- "Sisterhood"
col5b <- substring(survey.ids,5)
col5d <- "Pregnancy-related"
col5e <- 1                          # Marked as NA in input file, but we assume no adjustment
col6a <- pmdf.sisterhood            
col7a <- NA       # FILL IN IF YOU HAVE DENOMINATORS TO USE AS WEIGHTS
col8  <- "sisterhood.input.data.csv"
multiple.obs <- NA
duplicate <- NA
include <- NA

pm.mat <- data.frame(col1,col3,col4,col5,col5a,col5b,col5d,col5e,multiple.obs,duplicate,include,col6a,col7a,col8,row.names=NULL)
names(pm.mat) <-  c("iso.code","start.date","mid.date","end.date",
                   "data.type","source.detail","definition","adj.factor",
                   "multiple.obs","duplicate","include","pmdf.who",	"total.deaths.obs","pmdf.input.file")

write.csv(pm.mat,"../pm.dhsreports.csv",row.names=FALSE)