
# This file produces SAB trends.
# Alison Gemmill, 2015

#Import sab and country.info databases
#change file path names
sab <- read.csv("../sab2015input.csv")
info <-read.csv("../country.info_15Jul.csv")

#Extract vectors from sab matrix
sab$mid.date <- sab$start.date+(sab$end.date-sab$start.date)/2

sab.sort<- sab[order(sab$mid.date),]
sab.ctry.code <- unique(sab$iso.code)
sab.ctry.name <- unique(sab$country.name)

dates <- sort(unique(sab$mid.date))

sab.sort$value <- as.numeric(sab.sort$value)



#Extract vectors from country.info file
iso <- as.vector(info$iso.code)
in.study <- as.vector(info$in.study.12)
names(in.study) <- iso
cnames <- as.vector(info$country.name)


#Set up for logit model
sab.sort$value[which(sab.sort$value==1)] <- .99999
antilogit <- function(p){exp(p)/(1+exp(p))}
logit <- function(p){log(p/(1-p))}


#Create criteria for SAB models
obs.change <- tapply(sab.sort$value, sab.sort$iso.code, function(x) tail(x,n=1)-head(x,n=1))
min.year <- tapply(sab.sort$mid.date, sab.sort$iso.code,min)
obs.period <- tapply(sab.sort$mid.date, sab.sort$iso.code, function(x) tail(x,n=1)-head(x,n=1))
num.obs <-tapply(sab.sort$mid.date,sab.sort$iso.code,function(x) length(x))
obs.min <- tapply(sab.sort$value, sab.sort$iso.code, function(x) min(x))

#Hierarchical model criteria
uselme <- ( num.obs==1 | num.obs==2 | num.obs==3 |obs.min>=.95)
#iso[in.study][uselme]

iso.lme <- names(uselme)

#what countries are not in sab database?
nosabiso <- iso[in.study][!(iso[in.study]%in%sab.ctry.code)]
nosab <- rep(TRUE,length(nosabiso))
names(nosab) <- nosabiso

uselmenew <- c(uselme, nosab)

#merge in model criteria T/F column
lme.TF <- data.frame(iso=names(uselmenew),uselme=uselmenew)
sab.sort.2 <-merge(sab.sort, lme.TF, by.x="iso.code", by.y="iso", all=TRUE)
sab.sort<- sab.sort.2

#Add time since 1990 covariate
sab.sort$timesince1990 <- sab.sort$mid.date - 1990



#Hierarchical model
library(lme4)
#country.info <- read.csv("../Data/AuxData/country.info.csv")
sab.sort$region <- rep(NA, length(sab.sort$iso.code))
for (i in 1:length(sab.sort$iso.code)){
  sab.sort$region[i] <- paste(info$mdg.region.model.12[info$iso.code == as.character(sab.sort$iso.code[i])])
}
#data.frame(sab.sort$region, sab.sort$iso.code)
mod <- lmer(logit(value) ~ 1+timesince1990+(1+timesince1990 | iso.code) + (1+timesince1990  | region), data=sab.sort)  

sab.ctry.code <- unique(sab.sort$iso.code)
sab.ctry.name <- cnames[sab.ctry.code]


#Begin function
interp.fxn.sab <- function(dates){

  sab.est.all <- NA
 
  for(j in 1:length(sab.ctry.code)){

    sab.values <- sab.sort$value[which(sab.sort$iso.code==sab.ctry.code[j])]

    sab.mid.date <- sab.sort$mid.date[which(sab.sort$iso.code==sab.ctry.code[j])]
      
    dup.date <- sab.mid.date[duplicated(sab.mid.date)]
        
    sab.values[which(sab.mid.date==dup.date)] <-   mean(as.numeric(sab.values[which(sab.mid.date==dup.date)]))

    sab.values <- sab.values[!duplicated(sab.mid.date)]
    sab.mid.date <- sab.mid.date[!duplicated(sab.mid.date)]

    sab.data <- data.frame(sab.mid.date, sab.values)
    
    lll <- length(sab.mid.date)
    if(sum(is.na(sab.mid.date))>0) {lll <- 0}
    uselme <- unique(sab.sort$uselme[which(sab.sort$iso.code==sab.ctry.code[j])])
    
    if(lll>1 & uselme==FALSE ){
      sab.fit <- glm(sab.values ~ sab.mid.date, data=sab.data, family=quasibinomial)
      
      logit.sab.est <- coefficients(sab.fit)["(Intercept)"]+coefficients(sab.fit)["sab.mid.date"]*dates

      sab.est <- antilogit(logit.sab.est)

    }


    if((lll>=1 & uselme==TRUE) | lll==1)
    {
  ntemp <- which(sab.sort$iso.code==sab.ctry.code[j])[1]
  sab.est <- antilogit((
    fixef(mod)["(Intercept)"] + 
    + ranef(mod)$iso.code[sab.sort$iso.code[ntemp],"(Intercept)"]
    + ranef(mod)$region[sab.sort$region[ntemp],"(Intercept)"]
    + (dates-1990)*
        (fixef(mod)["timesince1990"]
         +ranef(mod)$region[sab.sort$region[ntemp],"timesince1990"]
         + ranef(mod)$iso.code[sab.sort$iso.code[ntemp],"timesince1990"]
          ))
        )                     
  }
  
  
  
   if(lll<1 & uselme==TRUE)
    { 	
  ntemp <- which(sab.sort$iso.code==sab.ctry.code[j])[1]
  sab.est <- antilogit((
    fixef(mod)["(Intercept)"] + 
    + ranef(mod)$region[sab.sort$region[ntemp],"(Intercept)"]
    + (dates-1990)*
        (fixef(mod)["timesince1990"]
        +ranef(mod)$region[sab.sort$region[ntemp],"timesince1990"]
          ))
        )                     
  }
  

    sab.est.dframe <- data.frame(rep(sab.ctry.code[j], length(dates)),dates, sab.est)
    
    sab.est.all <- rbind(sab.est.all, sab.est.dframe)
 
    sab.est.all <- sab.est.all[!is.na(sab.est.all$dates),]
  }

    return(sab.est.all)
}
    

ref.t.pred.1yr <- seq(1980.5, 2015.5, 1)
sab.est.pred.1yr <- interp.fxn.sab(ref.t.pred.1yr)
names(sab.est.pred.1yr) <- c("iso.code", "mid.date","sab.est")

#change output file path
write.csv(sab.est.pred.1yr, "../sab.estimates.csv")
